﻿using UnityEngine;
using UnityEngine.AI;
using UnityEngine.SceneManagement;

public class Enemy : MonoBehaviour
{

    NavMeshAgent agent;
    [SerializeField] float decisionDelay = 3f;
    [SerializeField] Transform objectToChase;
    [SerializeField] Transform[] waypoints;

    int currentWaypoint = 0;
    float timeSinceScan = 0;

    enum EnemyStates
    {
        Patrolling,
        Chasing
    }

    [SerializeField] EnemyStates currentState;

    void Start()
    {
        agent = GetComponent<NavMeshAgent>();
         if (currentState == EnemyStates.Patrolling) agent.SetDestination(waypoints[currentWaypoint].position);
    }

    void Update()
    {
        if (Vector3.Distance(transform.position, objectToChase.position) > 16f)
        {
            currentState = EnemyStates.Patrolling;
        }
        else
        {
            if(currentState == EnemyStates.Patrolling) Chase();
        }

        if (currentState == EnemyStates.Patrolling)
        {
            if (Vector3.Distance(transform.position, waypoints[currentWaypoint].position) <= 0.6f)
            {
                currentWaypoint++;
                if (currentWaypoint == waypoints.Length)
                {
                    currentWaypoint = 0;
                }
            }
            agent.SetDestination(waypoints[currentWaypoint].position);
        }
        if(currentState == EnemyStates.Chasing)
        {
            if(timeSinceScan >= decisionDelay)
            {
                timeSinceScan = 0;
                Chase();
            }
        }
        timeSinceScan += Time.deltaTime;
    }

    void Chase()
    {
        var path = new NavMeshPath();
        agent.CalculatePath(objectToChase.position, path);
        if(path.status == NavMeshPathStatus.PathComplete)
        {
            agent.SetDestination(objectToChase.position);
            currentState = EnemyStates.Chasing;
        }
        else
        {
            currentState = EnemyStates.Patrolling;
        }
    }

    private void OnCollisionEnter(Collision collision)
    {
        if (collision.transform.name == "Player")
        {
            SceneManager.LoadScene(SceneManager.GetActiveScene().buildIndex);
        }
    }

    private void OnDrawGizmos()
    {
        Gizmos.DrawWireSphere(transform.position, 8f);
    }
}
